" vim: tw=0

" This creates a keyword NOTICE and puts it in the highlight group called logNotice
:syn keyword logWarning WARNING
:syn keyword logCritical CRITICAL
:syn keyword logError ERROR
:syn keyword logAlert ALERT
:syn keyword logNotice NOTICE
:syn keyword logDebug DEBUG
:syn keyword logInfo INFO
:syn keyword logLog LOG

" This creates a match on the date and puts in the highlight group called logDate.  The
" nextgroup and skipwhite makes vim look for logTime after the match
:syn match logDate /^\d\{4}-\d\{2}-\d\{2}/ nextgroup=logTime skipwhite

" This creates a match on the time (but only if it follows the date)
:syn match logStarting /.*: Starting.*/ contained
:syn match logRunQuery /.* Run Query:.*/ contained
:syn match logTime /\d\{2}:\d\{2}:\d\{2}/
:syn match logFile /(\p*\.[ch]\.\d*)/ contains=logStarting,logRunQuery
:syn match logDiskDbEvent "\(DB_BAY_COMMIT\|DB_BAY_REMOVE\|DB_BAY_ROLE_ACCESS\|DB_DISK_PARTN_CHANGE\|DB_DISK_SCAN_START\|DB_DISK_SCAN_COMPLETE\|DB_DISK_REINSERT\|DB_DISK_ADD_CRYPT_MAP\|DB_DISK_SCANNING\|DB_DISK_REFRESH_CAPACITY\|DB_DISK_REFRESH\|DB_PLATFORM_DETECT\)"
:syn match logAuditEvent "\(AUDIT_EVNT_TASK_START\|AUDIT_EVNT_TASK_INFO_OPR\|AUDIT_EVNT_TASK_INFO_DRVT\|AUDIT_EVNT_TASK_INFO_DRV\|AUDIT_EVNT_TASK_INFO_ULK\|AUDIT_EVNT_TASK_INFO_VFY\|AUDIT_EVNT_TASK_INFO_WIP\|AUDIT_EVNT_TASK_INFO_ERR\|AUDIT_EVNT_TASK_INFO_SWIPE\|AUDIT_EVNT_TASK_INFO_FWIPE\|AUDIT_EVNT_TASK_INFO_CASE\|AUDIT_EVNT_TASK_INFO_SEG\|AUDIT_EVNT_TASK_INFO_DST\|AUDIT_EVNT_TASK_INFO_FILE\|AUDIT_EVNT_TASK_END\|AUDIT_EVNT_TASK_INFO_HASH\|AUDIT_EVNT_TASK_INFO_SGMNT\|AUDIT_EVNT_TASK_INFO_PUSH\|AUDIT_EVNT_TASK_INFO_FS_RESIZE\)"
:syn match logSysEvent "\(SYS_PLATFORM_DETECT\|SYS_IDENTIFY\|SYS_HASH\|SYS_STR_COMP\|SYS_AUTH_DONE\|SYS_DIT_RECONFIG\|SYS_IPADDR_ASSIGNED\|ISCSI_EVENT_CONNECT\|ISCSI_EVENT_DISCONNECT\)"
:syn match logCdbEvent "\(CDB_EVENT_TBL_ROW_MOD\)"
:syn match logSysError "\(SYS_ERROR_UNAUTH\|SYS_ERROR\)"
:syn match logJobError "\(JOB ERROR\|JOB ABORTED\)"
:syn match logUsbError "\(\*\*\*usb cmd failed\*\*\*\)"
:syn match logJobInfo "\(JOB PROGRESS\|JOB COMPLETED\|JOB COMMENCED\)"

" Now make them appear:
" Link just links logNotice to the colouring for error
hi link logNotice Error
hi link logDebug Identifier
hi link logInfo Identifier
hi link logLog Identifier
hi link logDate Comment
hi link logTime Conditional
hi link logFunction Comment
hi link logError Error
hi link logCritical Error
hi link logWarning Error
hi link logAlert Error

hi logFile ctermfg=magenta
hi logDiskDbEvent ctermfg=yellow cterm=bold
hi logAuditEvent ctermfg=blue cterm=bold
hi logSysEvent ctermfg=yellow cterm=bold
hi logCdbEvent ctermfg=yellow cterm=bold
hi logSysError ctermfg=red cterm=bold
hi logJobError ctermfg=white ctermbg=red
hi logUsbError ctermfg=red cterm=bold
hi logJobInfo ctermfg=green cterm=bold
hi logStarting ctermfg=red
hi logRunQuery ctermfg=lightgreen
