/* vim: set filetype=javascript: */

// This is a modified version of the github project stacktrace
// (https://github.com/stacktracejs/stacktrace.js). It has been 
// modified to remove the bloat and keep what we need

// Domain Public by Eric Wendelin http://eriwen.com/ (2008)
//                  Luke Smith http://lucassmith.name/ (2008)
//                  Loic Dachary <loic@dachary.org> (2008)
//                  Johan Euphrosine <proppy@aminche.com> (2008)
//                  Oyvind Sean Kinsey http://kinsey.no/blog (2010)
//                  Victor Homyakov <victor-homyakov@users.sourceforge.net> (2010)

var printStackTrace = function() {
	var p = new printStackTrace.implementation();
	try {
		this.undef();
	}
	catch (e) {
		var m = p.mode(e);
		if (m == 'other')
			return p.other(arguments.callee);
		else
			return p[m](e);
	}
}

printStackTrace.implementation = function() {
};

printStackTrace.implementation.prototype = {
	/**
	 * Mode could differ for different exception, e.g.
	 * exceptions in Chrome may or may not have arguments or stack.
	 *
	 * @return {String} mode of operation for the exception
	 */
	mode: function(e) {
		if ((e['arguments'] && e.stack) ||
			(e.stack && !e.fileName)) {
			return 'chrome';
		}

		if (e.stack && e.sourceURL) return 'safari';

		if (e.stack && e.number) return 'ie';

		if (e.stack && e.fileName) return 'firefox';

		return 'other';
	},

	/**
	 * Given an Error object, return a formatted Array based on Chrome's stack string.
	 *
	 * @param e - Error object to inspect
	 * @return Array<String> of function calls, files and line numbers
	 */
	chrome: function(e) {
		return (e.stack + '\n')
			.replace(/^[\s\S]+?\s+at\s+/, ' at ') // remove message
			.replace(/^\s+(at eval )?at\s+/gm, '') // remove 'at' and indentation
			.replace(/^([^\(]+?)([\n$])/gm, '{anonymous}() ($1)$2')
			.replace(/^Object.<anonymous>\s*\(([^\)]+)\)/gm, '{anonymous}() ($1)')
			.replace(/^(.+) \((.+)\)$/gm, '$1@$2')
			.split('\n')
			.slice(0, -1);
	},

	/**
	 * Given an Error object, return a formatted Array based on Safari's stack string.
	 *
	 * @param e - Error object to inspect
	 * @return Array<String> of function calls, files and line numbers
	 */
	safari: function(e) {
		return e.stack.replace(/\[native code\]\n/m, '')
			.replace(/^(?=\w+Error\:).*$\n/m, '')
			.replace(/^@/gm, '{anonymous}()@')
			.split('\n');
	},

	/**
	 * Given an Error object, return a formatted Array based on IE's stack string.
	 *
	 * @param e - Error object to inspect
	 * @return Array<String> of function calls, files and line numbers
	 */
	ie: function(e) {
		return e.stack
			.replace(/^\s*at\s+(.*)$/gm, '$1')
			.replace(/^Anonymous function\s+/gm, '{anonymous}() ')
			.replace(/^(.+)\s+\((.+)\)$/gm, '$1@$2')
			.split('\n')
			.slice(1);
	},

	/**
	 * Given an Error object, return a formatted Array based on Firefox's stack string.
	 *
	 * @param e - Error object to inspect
	 * @return Array<String> of function calls, files and line numbers
	 */
	firefox: function(e) {
		return e.stack.replace(/(?:\n@:0)?\s+$/m, '')
			.replace(/^(?:\((\S*)\))?@/gm, '{anonymous}($1)@')
			.split('\n');
	},

        // Safari 5-, IE 9-, and others
        other: function(curr) {
            var ANON = '{anonymous}', fnRE = /function\s*([\w\-$]+)?\s*\(/i, stack = [], fn, args, maxStackSize = 10;
            var slice = Array.prototype.slice;
            while (curr && curr['arguments'] && stack.length < maxStackSize) {
                fn = fnRE.test(curr.toString()) ? RegExp.$1 || ANON : ANON;
                args = slice.call(curr['arguments'] || []);
                stack[stack.length] = fn + '(' + this.stringifyArguments(args) + ')';
                try {
                    curr = curr.caller;
                } catch (e) {
                    stack[stack.length] = '' + e;
                    break;
                }
            }
            return stack;
        },

        /**
         * Given arguments array as a String, substituting type names for non-string types.
         *
         * @param {Arguments,Array} args
         * @return {String} stringified arguments
         */
        stringifyArguments: function(args) {
            var result = [];
            var slice = Array.prototype.slice;
            for (var i = 0; i < args.length; ++i) {
                var arg = args[i];
                if (arg === undefined) {
                    result[i] = 'undefined';
                } else if (arg === null) {
                    result[i] = 'null';
                } else if (arg.constructor) {
                    // TODO constructor comparison does not work for iframes
                    if (arg.constructor === Array) {
                        if (arg.length < 3) {
                            result[i] = '[' + this.stringifyArguments(arg) + ']';
                        } else {
                            result[i] = '[' + this.stringifyArguments(slice.call(arg, 0, 1)) + '...' + this.stringifyArguments(slice.call(arg, -1)) + ']';
                        }
                    } else if (arg.constructor === Object) {
                        result[i] = '#object';
                    } else if (arg.constructor === Function) {
                        result[i] = '#function';
                    } else if (arg.constructor === String) {
                        result[i] = '"' + arg + '"';
                    } else if (arg.constructor === Number) {
                        result[i] = arg;
                    } else {
                        result[i] = '?';
                    }
                }
            }
            return result.join(',');
        },
};	
